
#include "kernel.h"

void init_sem(void)
{
  int i;

  for( i=0; i<MAX_IPC; i++ ) {
    all_s[i].id=UNDEF;
  }
}

int sem_create( struct syscall *message, int index )
{
  int i;
  
  if ( message->arg1 < 1 ) return IPCERR_INVALIDID;

  for( i=0; i<MAX_IPC; i++ ) {
    if ( message->arg1 == all_s[i].id ) {
      if ( message->arg3 & IPC_EX ) return IPCERR_EXISTS;
      return i;
    }      
  }

  for( i=0; i<=MAX_IPC; i++ ) {
    if ( i == MAX_IPC ) {
      return UNDEF;
    }
    if ( all_s[i].id == UNDEF ) break;
  }

  all_s[i].id = message->arg1;
  all_s[i].val = message->arg2;
  all_s[i].r = 0;
  all_s[i].w = 0;
  all_s[i].blocked = 0;

  return i;
}

int sem_wait( struct syscall *message, int index )
{
  int i,w;

  i = message->arg1;

  if ( i < 0 || i >=MAX_IPC || all_s[ i ].id == UNDEF ) {
    return UNDEF;
  }
  
  if ( all_s[i].val > 0 ) {
    all_s[i].val--;
    return 0;                 // nebude blokovany
  }

  w = (all_s[i].w+1) % MAX_PROCESS;

  if ( w == all_s[i].r ) {    // plny queue
    return UNDEF;             // nebude blokovany
  }

  all_s[i].queue[w] = index;
  all_p[index].state = P_BLOCKED;  // blokovanie procesu, zabezpeci 
  all_s[i].blocked++;              // handler
  all_s[i].w = w;

  return 0;
}

int sem_signal( struct syscall *message, int index )
{
  int i,r,pi;

  i = message->arg1;

  if ( i < 0 || i >=MAX_IPC || all_s[ i ].id == UNDEF ) {
    return UNDEF;
  }
  
  if ( all_s[i].blocked == 0 ) {
    all_s[i].val++;
    return 0;
  }

  r = (all_s[i].r+1) % MAX_PROCESS;

  all_p[all_s[i].queue[r]].state = P_RUNNING;
  pi=all_s[i].queue[r];
  signal(all_p[pi].sockets[0],IPC_RECEIVE,&handler); //R0B0
  all_p[pi].syscall_return_value = 0;
  all_s[i].blocked--;
  all_s[i].r = r;

  return 0;
}

int sem_destroy( struct syscall *message, int index )
{
  int i,r;

  i = message->arg1;

  if ( i < 0 || i >=MAX_IPC || all_s[ i ].id == UNDEF ) {
    return UNDEF;
  }

  all_s[i].id = UNDEF;

  if ( all_s[i].blocked == 0 ) {
    return 0;
  }

  while( all_s[i].blocked-- != 0 ) {
    r = (all_s[i].r+1) % MAX_PROCESS;
    all_p[all_s[i].queue[r]].state = P_RUNNING;
  }

  return 0;
}
