
#ifndef __KERNEL_H
#define __KERNEL_H

#include "types.h"

#include <sys.h>
#include <ipc.h>
#include <scheduler.h>
#include <guard.h>
#include <pager.h>
#include <pgetab.h>
#include <frame.h>

#define MAX_PAGES 1024
#define MAX_PROCESS 10
#define MAX_CHILDS 5
#define MAX_IPC 10

#define P_BLOCKED 0
#define P_RUNNING 1
#define P_WAIT_FOR_CHILD 2
#define P_READY 3

#define CHILD_RUNNING 1
#define CHILD_EXITED 2

#define BUF_LENGTH 100

#define NUM_OF_CHANNELS 5

#define STACK_BASE (1024*1024*256)
#define CODE_SIZE  0xd65e

#define QUANTUM 64

#define MAP_NONE 	0
#define MAP_CODE	1
#define MAP_DATA	2
#define MAP_STACK   4	

#define U_FORK 1
#define U_KILL 2
#define U_EXIT 3
#define U_SEMGET 4
#define U_SEMWAIT 5
#define U_SEMSIGNAL 6
#define U_SEMDESTROY 7
#define U_MSGGET 8
#define U_MSGSEND 9
#define U_MSGRECEIVE 10
#define U_MSGDESTROY 11
#define U_WAIT 12
#define U_SHMGET 13
#define U_SHMDESTROY 14
#define U_GETACCT 15
#define U_GETMACCT 16
#define U_GETPID 17
#define U_MALLOC 18
#define U_FREE 19
#define U_MKDIR 20
#define U_LSDIR 21
#define U_CHDIR 22
#define U_CREATE 23
#define U_UNLINK 24
#define U_OPEN 25
#define U_CLOSE 26
#define U_WRITE 27
#define U_READ 28

#define MAX_FD 5

struct syscall
{
  int op;
  int arg1, arg2, arg3, arg4;
};

struct child
{
  int index;
  int state;
  int return_value;
};

struct process_s
{
  int state;
  int mapping;
  int syscall_return_value;
  sid_t sockets[NUM_OF_CHANNELS];
  pid_t pid;
  int pindex;
  struct child childs[MAX_CHILDS];
  addr_t eip;
  addr_t esp;
  unsigned int time_c;
  unsigned int mem_c;
  int priority;
  int qnt;
  int data_base;	//ukazovatel na dalsiu virtualnu adresu, od ktorej mozeme
  					//dynamicky malokovat
  int stack_base;   //posledne naalokovana stranka stacku
  int malloc_size;  //velkost pamate na malloc
  int fds[MAX_FD];
  int cwd;          // index do tabulky inodov
};

struct semaphore_s
{
  ipc_id id;
  int val;
  int blocked;
  int r,w;
  int queue[MAX_PROCESS];
};

/*
struct message_s
{
  ipc_id id;
  int free;
  char buffer[BUF_LENGTH];
  int r,w;
};
*/

struct shm_s
{
  ipc_id id;
  addr_t base;
  int size;
  int pid;
};

struct memory_page
{
  int base;
  int owner;
  int size;
  int priority;
  int attrib;
};

struct open_file
{
  int inode;
  int pos;
  int cnt;
  int flags;
  struct inode_t ino;
};

extern struct process_s all_p[MAX_PROCESS];
extern struct semaphore_s all_s[MAX_IPC];
//extern struct message_s all_m[MAX_IPC];
extern struct shm_s all_h[MAX_IPC];
extern struct memory_page pages[MAX_PAGES];
extern struct open_file all_f[ 32 ];
extern int stack_base;
extern int data_base_global;

extern sid_t init;
extern pid_t last_pid;
//extern int stack_base;

int socket2index( sid_t, int );
void init_sched();
int kp1( sid_t, int );
void schedule(void);

void u1(void);

void print_s( char *t );
void print_i( unsigned int n );
void printf( char *t, unsigned int n );
int kp1( sid_t s, int op );
int kp2( sid_t s, int op );
int kp3( sid_t s, int op );
int kp4( sid_t s, int op );
int handler( sid_t s, int op );
void init_process(void);
int process_fork( struct syscall *message, int i );
int process_exit( struct syscall *message, int i );
int process_wait( struct syscall *message, int index );
int process_getacct( struct syscall *message, int index );
int process_getmacct( struct syscall *message, int index );
int process_getpid( struct syscall *message, int index );
void run_u1(void);
void init_sem(void);
int sem_create( struct syscall *message, int index );
int sem_wait( struct syscall *message, int index );
int sem_signal( struct syscall *message, int index );
int sem_destroy( struct syscall *message, int index );
void init_shm(void);
addr_t ipc_shmget( struct syscall *message, int index );
int ipc_shmdestroy( struct syscall *message, int index );
void init_mem(void);
int mem_malloc( struct syscall *message, int index );
int mem_free( struct syscall *messa, int index );
void init_memory(void);
int stack_transform( int , int );
void init_ramdisk(void);
void mkfs(void);
int fs_mkdir(struct syscall *msg, int i);
int fs_lsdir(struct syscall *msg, int i);
int fs_chdir(struct syscall *msg, int i);
int mk_file( char *, int );
int unlink( char *, int );
void init_file( void );
int openf( char *, int, int );
int closef( int, int );
int write( int, char *, int, int );
int read( int, char *, int, int );

#endif





